/******************************************************************************
This version dated 20220422 is to update the benchmarking results for the years 
2006 to 2020, and examine sensitivy of results under alternative options explored
under the capitalisation review.

Option 1 is the base case with EBRIN opex data under the frozen 2014 CAM, which 
generates raw efficiency scores as per the 2021 ABR.

Option 2 is econometric modelling to directly capture capitalisaton practices. 
This invovles updating the international data (for computing opex/totex and 
opex/total cost ratios).  It is not covered in the version.

Option 3 uses opex data under current CAM in 2020.  This affects opex data for 
CIT/PCR/ERG. 

Option 4 uses adjusted opex series based on a common capitalisation ratio.  Four
series are generated for alternative periods (2006-2020 and 2012-2021 respectively)
and alterantive opex/capital measures (opex/totex and opex/total cost respectively). 

Option 5 uses adjusted opex series including capitalised corporate OHs.  

Source dataset on alternative opex series is provided in an email from Sasha 
Jergic dated 21 April 2022.

*******************************************************************************/


cd "H:\SuWu_2022\AER_opex\CAM\CAMs_20220422"

log using "vc-out-med-BM 20220422", replace
* AER VC CD & TLG LSE & SFA models, common time trend
* Australia & NZ data runs 2006-2020, Ontario data runs 2005-2019
* for NZ include Vector and Wellington separately for full period
* for Ontario include 4 Alectra component DNSPs for 2005-2016 and Alectra in 2017
* Australian DNSP identifiers are 1 ACT, 2 AGD, 3 CIT, 4 END, 5 ENX
* 6 ERG, 7 ESS, 8 JEN, 9 PCR, 10 SAP, 11 AND, 12 TND, 13 UED
version 14
clear
set more off
* set maxiter 2000

local tps FP SP


/*****************************************************************************
	1.  import the full dataset
******************************************************************************/

* import opex benchmarking data from EI 2021 ABR 
use Sourcedata\DNSPopex2.dta, clear
describe

sort eiid year
save data_1, replace


* import alternative opex serires for sensitivity analysis
import excel "Sourcedata/AER - Dx - Analysis - Capitalisation ratios - all DNSPs - 2021.xlsx", sheet("Calc|Summary") cellrange(B9:HA26) firstrow clear
describe
xpose, clear varname
drop _varname

rename v1 opex_1
rename v2 capex_1
drop v3 v4  /* capex from JEN and difference */
rename v5 auc
rename v6 MTFP
rename v7 MPFP_opex
rename v8 totex
rename v9 totcost
rename v10 opex_cod /* Opex - CAM at submission */
rename v11 opex_current /* Opex - current CAM backcasting - used for option 3*/
rename v12 custnum_1
rename v13 opex_adj_totex_fp /* adjusted opex series based on a common ratio opex/totex full period - used for option 4*/
rename v14 opex_adj_totex_sp
rename v15 opex_adj_totcost_fp
rename v16 opex_adj_totcost_sp
rename v17 opex_cap /* Opex + capitalised corporate overheads - used for option 5*/

gen eiid =.
gen year =.
local k=1
forvalues i=1/13 { 
	
	forvalues j=1/16 {
		replace eiid = `i' if _n == `k'
		replace year = 2005 + `j' if _n ==`k'
		local ++k
	
	}

}

tab2 eiid year

list if _n <=3

** correct data errors with ERG 2020 
replace opex_cod=398134.063 if eiid ==6 & year == 2020
replace opex_current=398134.063 if eiid ==6 & year == 2020
replace opex_cap=opex_cap + 398134.063 - 390182.790 if eiid ==6 & year == 2020
drop if year ==2021 /* drop opex series in 2021 for Australian DNSPs - the data are to be subject to further checking */

sort eiid year
save opex_alt, replace


* combining the two alternative opex series with opex benchmarking data
use data_1, clear
merge 1:1 eiid year using opex_alt, sorted
tab _merge
drop _merge

format opex* %9.2fc
save data_2, replace


*checking opex/customer number series differences
use data_2, clear
keep if eiid <=13

gen opex_diff1 = opex_1 - opex
format opex_diff1 %9.2fc
list eiid year opex opex_1 opex_diff1 if abs(opex_diff1) > 0.1, clean

gen opex_diff2 = opex_cod- opex_1
gen opex_diff2_pct = (opex_cod/opex_1 - 1)*100
format opex_diff2 %9.2fc
format opex_diff2_pct %9.2f
list eiid year opex opex_1 opex_cod opex_diff2 opex_diff2_pct if abs(opex_diff2) > 0.1, clean

gen opex_diff3 = opex_current - opex_cod
gen opex_diff3_pct = (opex_current/opex_cod - 1)*100
format opex_diff3 %9.2fc
format opex_diff3_pct %9.2f
list eiid year opex_1 opex_cod opex_current opex_diff3 opex_diff3_pct if abs(opex_diff3) > 0.1, clean

* compare opex_4 series
gen opex_adj_totex_ratio = opex_adj_totex_fp / opex_adj_totex_sp
gen opex_adj_totcost_ratio = opex_adj_totcost_fp / opex_adj_totcost_sp
summarize opex_adj_totex_ratio opex_adj_totcost_ratio

** opex_5 smaller than opex_3 for CIT/PCR 2009-2015 (due to difference with recast data)
** opex_4 small than opex_3 for SAP 2012-2020 due to its negative capitalised corporate OH reported
gen opex_diff5 = opex_cap - opex_current
gen opex_diff5_pct = (opex_cap/opex_current -1)*100
format opex_diff5 %9.2fc
format opex_diff5_pct %9.2f
list eiid year opex_1 opex_current opex_cap opex_diff5 opex_diff5_pct if abs(opex_diff5) > 0.1 & year >=2009, clean

gen custnum_diff = custnum - custnum_1
list eiid year custnum custnum_1 custnum_diff if abs(custnum_diff) >=1, clean

by eiid, sort: summarize opex opex_1 opex_cod opex_current opex_cap

save data_2_Aust, replace



/*****************************************************************************
	2.  Generating varaibles for modelling 
******************************************************************************/


use data_2, clear

summarize
correlate custnum circlen rmdem

xtset eiid year

* construct dummy variables
tabulate country, gen(jur)
tabulate eiid, gen(d)
drop d14-d69 /* to check */


* adjust to calendar year
generate yr=year
replace yr=yr+0.5 if eiid==3
replace yr=yr+0.5 if eiid==8
replace yr=yr+0.5 if eiid==9
replace yr=yr+0.5 if eiid==11
replace yr=yr+0.5 if eiid==13
replace yr=yr-0.25 if jur2==1
replace yr=yr+0.5 if jur3==1

* adjust base to 2012
replace propex=propex/1.239047935 if jur1==1
replace propex=propex/1.221665248 if jur2==1
replace propex=propex/1.160126855 if jur3==1

* adjust for PPPs in 2012
* Australia PPP = 1.527404363
* NZ PPP = 1.482877481
* Canada PPP = 1.249076963
* converting into Australian Dollar base
replace propex=propex*1.482877481/1.527404363 if jur2==1
replace propex=propex*1.249076963/1.527404363 if jur3==1

* create alternative opex series for DNSPs in NZ and ONT
foreach i in _1 _current _adj_totex_fp _adj_totex_sp _adj_totcost_fp _adj_totcost_sp _cap {

	replace opex`i' = opex if jur1==0

} 

foreach i in _1 _current _adj_totex_fp _adj_totex_sp _adj_totcost_fp _adj_totcost_sp _cap {

	generate vc`i'=opex`i'/propex
	generate lvc`i'=log(vc`i')

} 

generate ly2=log(custnum)
generate ly3=log(circlen)
generate ly4=log(rmdem)

generate z1=shareugc

generate lz1=log(z1)

save data_3, replace


/*****************************************************************************
	3.  Prepare full period and short period datasets
******************************************************************************/

use data_3, clear

rename lvc_current lvc_3
rename lvc_adj_totex_fp lvc_4a
rename lvc_adj_totcost_fp lvc_4b
rename lvc_cap lvc_5 

save data_fp, replace

* delete years prior to 2012 (prior to 2011 for Ontario)
generate yr1=year
replace yr1=yr1+1.0 if jur3==1
drop if yr1 < 2012

drop yr1

* update opex series with short period common ratio
replace lvc_4a = lvc_adj_totex_sp
replace lvc_4b = lvc_adj_totcost_sp

save data_sp, replace

* mean correct output data
foreach tp of local tps {
	
	use data_`tp', clear
	qui reg ly2
	predict mly2
	replace ly2=ly2-mly2
	qui reg ly3
	predict mly3
	replace ly3=ly3-mly3
	qui reg ly4
	predict mly4
	replace ly4=ly4-mly4

	generate ly22=ly2*ly2/2
	generate ly33=ly3*ly3/2
	generate ly44=ly4*ly4/2
	generate ly23=ly2*ly3
	generate ly24=ly2*ly4
	generate ly34=ly3*ly4

	save data_`tp'_1, replace

}


/*****************************************************************************
	4.  Econometric modelling with alternative datasets 
******************************************************************************/

foreach tp of local tps {
	
	use data_`tp'_1, clear
	
	foreach i in _1 _3 _4a _4b _5 {
		
		* if option 5 for full period
		if "`tp'" == "FP" & "`i'"=="_5" {
		
			display "NOT to use opex series `i' for `tp'! "
		
			}
	
		else {
		
		display "using opex series `i': "
		rename lvc`i' lvc
		* models with 3 outputs:

		* regress lvc ly2-ly4 lz1 yr jur2 jur3
		* regress lvc ly2-ly4 ly22 ly23 ly24 ly33 ly34 ly44 lz1 yr jur2 jur3
		* regress lvc ly2-ly4 lz1 yr jur2 jur3 d2-d13
		* regress lvc ly2-ly4 ly22 ly23 ly24 ly33 ly34 ly44 lz1 yr jur2 jur3 d2-d13

		** LSE CD model
		xtpcse lvc ly2-ly4 lz1 yr jur2 jur3 d2-d13, c(a) het
		estimates store LSECD`i'
		
		** LSE TLG model
		xtpcse lvc ly2-ly4 ly22 ly23 ly24 ly33 ly34 ly44 lz1 yr jur2 jur3 d2-d13, c(a) het
		estimates store LSETLG`i'
		
		* monotonicity tests
		gen el2 = _b[ly2] + _b[ly22]*ly2 + _b[ly23]*ly3 + _b[ly24]*ly4
		gen el3 = _b[ly3] + _b[ly23]*ly2 + _b[ly33]*ly3 + _b[ly34]*ly4
		gen el4 = _b[ly4] + _b[ly24]*ly2 + _b[ly34]*ly3 + _b[ly44]*ly4
		gen el_total = el2+el3+el4
		
		display "LSE TLG Elasticities using opex series `i':"
		tabstat el2 el3 el4 el_total, by(country) statistics(mean)
		tabstat el2 el3 el4 el_total if eiid <=13, by(eiid) statistics(mean) 
		
		display "LSE TLG Monotonicity Violations using opex series `i': "
		gen mon2 = cond(el2 < 0,1,0)
		gen mon3 = cond(el3 < 0,1,0)
		gen mon4 = cond(el4 < 0,1,0)
		gen mon = cond(mon2+mon3+mon4 >=1, 1, 0)
		tabstat mon2 mon3 mon4 mon, by(country) statistics(mean)
		tabstat mon2 mon3 mon4 mon if eiid <=13, by(eiid) statistics(mean) 
		export excel country eiid year el2 el3 el4 el_total mon2 mon3 mon4 mon using Deriveddata/TLG_outputelasticities.xlsx, sheet("opex`i'_`tp'") cell(a6) sheetmodify
		
		rename mon mon_LSE`i'
		
		drop el2 el3 el4 el_total mon2 mon3 mon4 

		* SFA CD model
		qui reg lvc ly2-ly4 lz1 yr jur2 jur3
		matrix b0 = e(b), .1, ln(e(rmse)^2) , .1
		xtfrontier lvc ly2-ly4 lz1 yr jur2 jur3, ti cost from(b0, copy)
		estimates store SFACD`i'
		
		predict ci,te
		frontier_teci ci
		generate ceff=1/ci
		generate ceff_l=1/ci_u
		generate ceff_u=1/ci_l
		list dnsp ceff ceff_l ceff_u if year == 2019, clean
		summarize dnsp year ceff ceff_l ceff_u

		generate SFACD`i'=ceff

		drop ci ci_u ci_l ceff ceff_l ceff_u

		* SFA TLG model		
		qui reg lvc ly2-ly4 ly22 ly23 ly24 ly33 ly34 ly44 lz1 yr jur2 jur3
		
		if "`tp'" == "FP" & "`i'" =="_4b" {
				matrix b0 = e(b), .1, ln(e(rmse)^2) , .2
			} 
			else {
				matrix b0 = e(b), .1, ln(e(rmse)^2) , .1
			}
		
		xtfrontier lvc ly2-ly4 ly22 ly23 ly24 ly33 ly34 ly44 lz1 yr jur2 jur3, ti cost from(b0, copy)
		estimates store SFATLG`i'
		
		predict ci,te
		frontier_teci ci
		generate ceff=1/ci
		generate ceff_l=1/ci_u
		generate ceff_u=1/ci_l
		list dnsp ceff ceff_l ceff_u if year ==2019, clean
		summarize dnsp year ceff ceff_l ceff_u

		generate SFATLG`i'=ceff
		
		summarize dnsp year SFACD`i' SFATLG`i'
		
		drop ci ci_u ci_l ceff ceff_l ceff_u

		* monotonicity tests
		gen el2 = _b[ly2] + _b[ly22]*ly2 + _b[ly23]*ly3 + _b[ly24]*ly4
		gen el3 = _b[ly3] + _b[ly23]*ly2 + _b[ly33]*ly3 + _b[ly34]*ly4
		gen el4 = _b[ly4] + _b[ly24]*ly2 + _b[ly34]*ly3 + _b[ly44]*ly4
		gen el_total = el2+el3+el4
		
		display "SFA TLG Elasticities using opex series `i':"
		tabstat el2 el3 el4 el_total, by(country) statistics(mean)
		tabstat el2 el3 el4 el_total if eiid <=13, by(eiid) statistics(mean) 
		
		display "SFA TLG Monotonicity Violations using opex series `i': "
		gen mon2 = cond(el2 < 0,1,0)
		gen mon3 = cond(el3 < 0,1,0)
		gen mon4 = cond(el4 < 0,1,0)
		gen mon = cond(mon2+mon3+mon4 >=1, 1, 0)
		tabstat mon2 mon3 mon4 mon, by(country) statistics(mean)
		tabstat mon2 mon3 mon4 mon if eiid <=13, by(eiid) statistics(mean) 
		export excel country eiid year el2 el3 el4 el_total mon2 mon3 mon4 mon using Deriveddata/TLG_outputelasticities.xlsx, sheet("opex`i'_`tp'") cell(o6) sheetmodify
		
		rename mon mon_SFA`i'
		drop el2 el3 el4 el_total mon2 mon3 mon4 
		
		rename lvc lvc`i'
		
		}
	
	}	
	
	save data_`tp'_2, replace	
	
	
	/* exporting efficiency results  */
	use data_`tp'_2, clear
	keep dnsp year opex*  propex custnum circlen rmdem vc* lvc* SFACD* SFATLG*
	collapse SFACD* SFATLG*, by(dnsp)
	rename dnsp EIID
	save sfa_es_`tp'_results, replace
	export excel Deriveddata/Results.xlsx, sheet("ES_`tp'") cell(b3) firstrow(var) sheetmodify

	
	/* Exporting regression results to MS excel   */
	* compare regression results
	estimates table LSECD* LSETLG* SFACD* SFATLG*, b(%9.3f) t(%5.3f) stats(N r2 r2_a ll chi2)
	
	* report regression results in excel
	esttab LSECD* LSETLG* SFACD* SFATLG* using Deriveddata/Table_`tp'.csv, scalars(N) b(%9.3f) t(%5.3f) wide nostar plain r2(%5.3f) ar2(%5.3f) title(Table: Benchmarking - `tp', Alternative Opex Series) /*mtitle()*/ replace	
	
	* report monotonicty result in excel - summary table of monotonicity results for Australian DNSPs
	use data_`tp'_2, clear
	keep if eiid<=13
	keep eiid year mon*
	order eiid year mon_LSE_* mon_SFA_*
	
	collapse mon*, by(eiid)
	rename mon_* *
	save results_`tp'_mon_aus, replace
	drop eiid
	
	export excel Deriveddata/Results.xlsx, sheet("Monotonicity_`tp'") cell(b5) firstrow(var) sheetmodify
	
} 


log close 
